/*
 * "Portable" interface to system-dependent terminal mode control.
 *
 * This module is used by the calling program to test/set/clear
 * such terminal parameters as echo, buffered i/o, tab expansion,
 * etc.  Sorry, but some of these modes are derived from historic
 * usage on UNIX, and may have no reasonable analog on the target
 * system.  In such a case, test should return "off", and set should
 * return "error" for operations concerning the aberrant mode.  It
 * is up to the using program to check return values.
 *
 * The routines supplied are:
 *
 *	tstterm(mode)		- returns on/off status
 *	setterm(mode,action)	- returns success/failure
 *
 * The modes which are handled are:
 *
 *	TM_ECHO		- echo keyboard input characters to screen.
 *	TM_XTABS	- tab is mapped to spaces to the screen.
 *	TM_CRMOD	- map CR -> LF from keyboard.
 *	TM_CBREAK	- suppress os buffering.
 *	TM_RAW		- suppress all processing.  if TM_RAW is
 *			  clear, UNIX signal characters work, LF
 *			  generates both CR and LF to the screen,
 *			  and input buffering is enabled (unless
 *			  TM_CBREAK is set).  If TM_RAW is set,
 *			  no mapping occurs, no buffering takes
 *			  place, and each key represents data.
 *			  This mode, if set, overrides the others.
 *
 * Actions for setterm() include:
 *
 *	TM_SET		- set the named mode, if supported.
 *	TM_CLEAR	- reset the named mode, if supported.
 *	TM_SAVE		- save current mode vector in hidden location.
 *	TM_RESTORE	- restore mode vector saved via TM_SAVE.
 */

#include "term.h"

/*
 * V7 UNIX version of terminal mode control.
 *
 * this module is also believed to be compatible with:
 *
 *	XENIX
 *	DYNIX
 *	OS/1 (on Onyx C8002)
 *	ON-PLUS
 *	ONIX
 */

#include <sgtty.h>

static struct sgttyb ttyctl;	/* work buffer for setterm(), tstterm() */
static struct sgttyb ttysave;	/* buffer for TM_SAVE, TM_RESTORE */

/*
 * table to map from term mode (eg TM_CRMOD) to sgtty flag bit.
 * the ordering is based on the define values in term.h.  don't
 * mess with it!
 */
static int sgflag[] =
	{
	RAW,
	CBREAK,
	ECHO,
	CRMOD,
	XTABS,
	LCASE,
	};

int setterm(action,mode)
	int action, mode;
	{
	if (action == TM_SAVE)		/* action for preserve state */
		{
		gtty(2,&ttysave);
		}
	else if (action == TM_RESTORE)	/* action for restore state */
		{
		stty(2,&ttysave);
		}
	else if (0 <= mode && mode < TM_NUMMODES) /* valid mode */
		{
		gtty(2,&ttyctl);

		if (action == TM_SET)
			ttyctl.sg_flags |=  sgflag[mode];
		else if (action == TM_CLEAR)
			ttyctl.sg_flags &=~ sgflag[mode];
		else
			return(-1); /* invalid action */

		return(stty(2,&ttyctl));
		}
	else
		return(-1);			/* invalid mode */
	}

int tstterm(mode)
	int mode;
	{
	if (TM_RAW <= mode && mode <= TM_XTABS) /* valid mode */
		{
		gtty(2,&ttyctl);
		return( ttyctl.sg_flags & sgflag[mode] );
		}
	else
		return(0);			/* invalid mode */
	}
